@@ -1,6 +1,6 @@
 #ifndef GCC_TM_H
 #define GCC_TM_H
-#define TARGET_CPU_DEFAULT (TARGET_CPU_generic)
+#define TARGET_CPU_DEFAULT ("generic-armv7-a")
 #ifndef LIBC_GLIBC
 # define LIBC_GLIBC 1
 #endif
@@ -10,9 +10,16 @@
 #ifndef LIBC_BIONIC
 # define LIBC_BIONIC 3
 #endif
+#ifndef LIBC_MUSL
+# define LIBC_MUSL 4
+#endif
+#ifndef DEFAULT_FLAG_PIE
+# define DEFAULT_FLAG_PIE 1
+#endif
 #ifdef IN_GCC
 # include "options.h"
 # include "insn-constants.h"
+# include "config/vxworks-dummy.h"
 # include "config/dbxelf.h"
 # include "config/elfos.h"
 # include "config/arm/elf.h"
@@ -20,6 +27,7 @@
 # include "config/arm/openbsd1.h"
 # include "config/arm/arm.h"
 # include "config/openbsd.h"
+# include "config/openbsd-stdint.h"
 # include "config/openbsd-libpthread.h"
 # include "config/arm/openbsd.h"
 # include "config/initfini-array.h"
