@@ -18,10 +18,12 @@
 #ifdef IN_GCC
 # include "options.h"
 # include "insn-constants.h"
+# include "config/vxworks-dummy.h"
 # include "config/rs6000/secureplt.h"
 # include "config/rs6000/rs6000.h"
 # include "config/dbxelf.h"
 # include "config/elfos.h"
+# include "config/gnu-user.h"
 # include "config/openbsd.h"
 # include "config/openbsd-stdint.h"
 # include "config/openbsd-libpthread.h"
