@@ -15,6 +15,9 @@
 #ifndef DEFAULT_FLAG_PIE
 # define DEFAULT_FLAG_PIE 1
 #endif
+#ifndef USE_IX86_FRAME_POINTER
+# define USE_IX86_FRAME_POINTER 1
+#endif
 #ifdef IN_GCC
 # include "options.h"
 # include "insn-constants.h"
